;
; Copyright (c) 1993 by Tipton Cole + Company
;
; Permission is granted to make and distribute verbatim copies of this
; software provided that the copyright notice and this permission notice
; are preserved on all copies.
;
; Permission is granted to copy and distribute modified versions of this
; software under the conditions for verbatim copying, provided that the
; entire resulting derived work is distributed under the terms of a
; permission notice identical to this one.
;
; Permission is granted to embed this software (verbatim or modified) in
; other programs provided that the legend "Portions copyrighted (c) 1993 
; by Tipton Cole + Company" appears along with any copyright notice
; contained in or displayed by the other programs.
;
; This software is provided "as is" and without any express or implied
; warranties, including, without limitation, the implied warranties of
; merchantability and fitness for a particular purpose.
;
CREATELIB "EventLib"
PROC CLOSED EventLib.Prepare.u(type.a, subtype.a, handler.p)
    USEVARS EventLib.contingency_list.pyg
    PRIVATE ndx.a       ; index into the contingency list

    IF NOT ISASSIGNED(EventLib.contingency_list.pyg) THEN
        DYNARRAY EventLib.contingency_list.pyg[]
    ENDIF
    SWITCH
        CASE (type.a = "IDLE") OR (type.a = "ALL") :
            subtype.a = ""
        CASE (type.a = "KEY") AND (subtype.a <> "ALL") :
            subtype.a = ASC(subtype.a)      ; temporarily not alphanumeric
    ENDSWITCH

    ndx.a = STRVAL(type.a) + ":" + STRVAL(subtype.a)
    EventLib.contingency_list.pyg[ndx.a] = handler.p
ENDPROC
PROC CLOSED EventLib.Purge.u(type.a, subtype.a)
    USEVARS EventLib.contingency_list.pyg
    PRIVATE ndx.a       ; index into the contingency list

    IF NOT ISASSIGNED(EventLib.contingency_list.pyg) THEN
        DYNARRAY EventLib.contingency_list.pyg[]
    ENDIF
    SWITCH
        CASE (type.a = "IDLE") OR (type.a = "ALL") :
            subtype.a = ""
        CASE (type.a = "KEY") AND (subtype.a <> "ALL") :
            subtype.a = ASC(subtype.a)      ; temporarily not alphanumeric
    ENDSWITCH

    ndx.a = STRVAL(type.a) + ":" + STRVAL(subtype.a)
    RELEASE VARS EventLib.contingency_list.pyg[ndx.a]
ENDPROC
PROC EventLib.Suspend.u()
;   USEVARS EventLib.contingency_list.pyg

    PROC WaitProc.s(event_type.a, event_record.vy, cycle.n)

        IF (event_type.a <> "EVENT") THEN    ; we have a trigger event
            event_record.vy["ACTUALTYPE"] = event_record.vy["TYPE"]
            event_record.vy["TYPE"] = "TRIGGER"
            event_record.vy["TRIGGER"] = event_type.a
            event_record.vy["CYCLE"] = cycle.n
        ENDIF
        RETURN EventLib.Cause.s(event_record.vy)
    ENDPROC

    IF NOT ISASSIGNED(EventLib.contingency_list.pyg) THEN
        DYNARRAY EventLib.contingency_list.pyg[]
    ENDIF
    WAIT WORKSPACE
        PROC "WaitProc.s"
        ALL
    ENDWAIT
ENDPROC
PROC EventLib.Cause.s(event.vy)
;   USEVARS EventLib.contingency_list.pyg
    PRIVATE type.a,         ; used to create index
            subtype.a,      ;   into contingency list
            ndx.a           ; the aforementioned index

    IF NOT ISASSIGNED(EventLib.contingency_list.pyg) THEN
        DYNARRAY EventLib.contingency_list.pyg[]
    ENDIF
    type.a = event.vy["TYPE"]
    SWITCH
        CASE (type.a = "MOUSE") :
            subtype.a = event.vy["ACTION"]
        CASE (type.a = "KEY") :
            subtype.a = STRVAL(event.vy["KEYCODE"])
        CASE (type.a = "MESSAGE") :
            subtype.a = event.vy["MESSAGE"]
        CASE (type.a = "TRIGGER") :
            subtype.a = event.vy["TRIGGER"]
        CASE (type.a = "IDLE") :
            subtype.a = ""
        OTHERWISE :
            IF ISASSIGNED(event.vy["SUBTYPE"]) THEN
                subtype.a = event.vy["SUBTYPE"]
            ELSE
                subtype.a = ""
            ENDIF
    ENDSWITCH
    IF (NOT ISASSIGNED(
            EventLib.contingency_list.pyg[type.a + ":" + subtype.a])) THEN
        subtype.a = "ALL"
        IF (NOT ISASSIGNED(
                EventLib.contingency_list.pyg[type.a + ":" + subtype.a])) THEN
            type.a = "ALL"
            subtype.a = ""
        ENDIF
    ENDIF
    ndx.a = type.a + ":" + subtype.a
    IF (ISASSIGNED(EventLib.contingency_list.pyg[ndx.a])) THEN
        EXECUTE EventLib.contingency_list.pyg[ndx.a] + "(event.vy)"
        RETURN RETVAL
    ELSE        ; no appropriate handler
        RETURN 0
    ENDIF
ENDPROC
PROC CLOSED EventLib.ProcessEvent.s(unused_event.vy)
    RETURN 0
ENDPROC
PROC CLOSED EventLib.IgnoreEvent.s(unused_event.vy)
    RETURN 1
ENDPROC
PROC CLOSED EventLib.Resume.s(unused_event.vy)
    RETURN 2
ENDPROC
PROC CLOSED EventLib.Reset.u()
    USEVARS EventLib.contingency_list.pyg

    DYNARRAY EventLib.contingency_list.pyg[]    ; erase what was there before
ENDPROC
PROC CLOSED EventLib.SaveContingencies.u(saved_list.pyg)
    USEVARS EventLib.contingency_list.pyg
    PRIVATE index.a     ; index into both dynamic arrays

    IF NOT ISASSIGNED(EventLib.contingency_list.pyg) THEN
        DYNARRAY EventLib.contingency_list.pyg[]
    ENDIF
    FOREACH index.a IN saved_list.pyg
        RELEASE VARS saved_list.pyg[index.a]
    ENDFOREACH
    FOREACH index.a IN EventLib.contingency_list.pyg
        saved_list.pyg[index.a] = EventLib.contingency_list.pyg[index.a]
    ENDFOREACH
ENDPROC
PROC CLOSED EventLib.RestoreContingencies.u(saved_list.pyg)
    USEVARS EventLib.contingency_list.pyg
    PRIVATE index.a     ; index into both dynamic arrays

    IF NOT ISASSIGNED(EventLib.contingency_list.pyg) THEN
        DYNARRAY EventLib.contingency_list.pyg[]
    ENDIF
    FOREACH index.a IN EventLib.contingency_list.pyg
        RELEASE VARS EventLib.contingency_list.pyg[index.a]
    ENDFOREACH
    FOREACH index.a IN saved_list.pyg
        EventLib.contingency_list.pyg[index.a] = saved_list.pyg[index.a]
    ENDFOREACH
ENDPROC
PROC CLOSED EventLib.GetHandler.a(type.a, subtype.a)
    USEVARS EventLib.contingency_list.pyg
    PRIVATE ndx.a       ; index into the contingency list

    IF NOT ISASSIGNED(EventLib.contingency_list.pyg) THEN
        DYNARRAY EventLib.contingency_list.pyg[]
    ENDIF
    SWITCH
        CASE (type.a = "IDLE") OR (type.a = "ALL") :
            subtype.a = ""
        CASE (type.a = "KEY") AND (subtype.a <> "ALL") :
            subtype.a = ASC(subtype.a)      ; temporarily not alphanumeric
    ENDSWITCH

    ndx.a = STRVAL(type.a) + ":" + STRVAL(subtype.a)
    IF (ISASSIGNED(EventLib.contingency_list.pyg[ndx.a])) THEN
        RETURN EventLib.contingency_list.pyg[ndx.a]
    ELSE
        RETURN ""
    ENDIF
ENDPROC
WRITELIB "EventLib" EventLib.Prepare.u
WRITELIB "EventLib" EventLib.Purge.u
WRITELIB "EventLib" EventLib.Suspend.u
WRITELIB "EventLib" EventLib.Cause.s
WRITELIB "EventLib" EventLib.ProcessEvent.s
WRITELIB "EventLib" EventLib.IgnoreEvent.s
WRITELIB "EventLib" EventLib.Resume.s
WRITELIB "EventLib" EventLib.Reset.u
WRITELIB "EventLib" EventLib.SaveContingencies.u
WRITELIB "EventLib" EventLib.RestoreContingencies.u
WRITELIB "EventLib" EventLib.GetHandler.a


